from exceptions.invalid_parameter import InvalidParameters
from paystack.gateways.gateway import BaseGateway


class TransactionGateway(BaseGateway):
    gateway_path = "/transaction"

    def initiate(self, params):
        payload = dict(**params)
        if "amount" not in payload:
            raise InvalidParameters("Amount to be charged needs to be specified")
        if "email" not in payload:
            raise InvalidParameters("Customer email is required")
        if "channels" in payload and not isinstance(payload["channels"], list):
            raise InvalidParameters("Channels field needs to be an Array of strings")
        return self.post(f"{self.gateway_path}/initialize", params)

    def verify(self, reference):
        response = self.get(f"{self.gateway_path}/verify/{reference}")
        return response

    def list(self, query_params=None):
        return self.get(self.gateway_path, query_params)

    def retrieve(self, txn_id):
        response = self.get(f"{self.gateway_path}/{txn_id}")
        return response

    def authorize(self, params):
        """
        All authorizations marked as reusable can be charged with this endpoint whenever you need to recieve payments.
        :param params:
        :return:
        """
        payload = dict(**params)
        if "authorization_code" not in payload:
            raise InvalidParameters("Customer authorization code is required")
        if "email" not in payload:
            raise InvalidParameters("Customer email is required")
        if "amount" not in payload:
            raise InvalidParameters("Amount to be charged is required")
        response = self.post(f"{self.gateway_path}/charge_authorization", payload)
        return response

    def check_authorization(self, params):
        """
        All mastercard and visa authorizations can be checked with this endpoint to know if they have funds for the payment you seek.

        This endpoint should be used when you do not know the exact amount to charge a card when rendering a service. It should be used to check if a card has enough funds based on a maximum range value. It is well suited for:

        Ride hailing services
        Logistics services
        :param params: amount<required>, email<required>, authorization_code<required>, currency<optional>
        :return:
        You shouldn't use this endpoint to check a card for sufficient funds if you are going to charge the user immediately. This is because we hold funds when this endpoint is called which can lead to an insufficient funds error.
        """
        response = self.post(f"{self.gateway_path}/check_authorization", params)
        return response

    def get_transaction_history(self, txn_id_or_ref):
        """
        View the timeline of a transaction
        :param txn_id_or_ref: Transaction id or reference
        :return:
        """
        response = self.get(f"{self.gateway_path}/timeline/{txn_id_or_ref}")
        return response

    def get_transaction_totals(self, params=None):
        """
        Returns total amount received on your account
        :param params:
        """
        return self.get(f"{self.gateway_path}/totals", params)

    def export_transactions(self, query_params=None):
        """
        Generates a report of all transactions carried out on your account :param query_params: Url query params to
        filter by, Check the documentation for more details on the available url parameters
        """
        return self.get(f"{self.gateway_path}/export", query_params)

    def partial_charge(self, params):
        """
        Retrieve part of a payment from a customer
        :param params: '{ authorization: "AUTH_72btv547", currency: "NGN", amount: "20000" }'
        """
        return self.post(f"{self.gateway_path}/partial_debit", params)



from paystack.gateways.gateway import BaseGateway
from paystack.exceptions.invalid_parameter import InvalidParameters


class TransferControl(BaseGateway):
    gateway_path = "/balance"
    control_path = "/transfer"

    def check_balance(self):
        return self.get(self.gateway_path)

    def balance_ledger(self):
        return self.get(f"{self.gateway_path}/ledger")

    def resend_otp(self, params):
        """
        Creates a new recipient. An duplicate account number will lead to the retrieval of the existing record.
        :param params: transfer_code (required) - Transfer code
        reason (required) - either resend_otp or transfer
        """
        payload = dict(**params)
        self.validate(params)
        if "transfer_code" not in payload:
            raise InvalidParameters("Transfer code is required")
        if "reason" not in payload or payload["reason"] == "":
            payload["reason"] = "resend_otp"
        return self.post(f"{self.control_path}/resend_otp", payload)

    def disable_otp(self):
        """
        In the event that you want to be able to complete transfers programmatically without use of OTPs,
        this endpoint helps disable that….with an OTP. No arguments required. You will get an OTP. In the event that
        you want to be able to complete transfers programmatically without use of OTPs, this endpoint helps disable
        that….with an OTP. No arguments required. An OTP is sent to you on your business phone.
        """
        return self.post(f"{self.control_path}/disable_otp", params=None)

    def finalize_disable(self, params):
        """
        :param params: otp (required) - OTP sent to business phone to verify disabling OTP requirement
        """
        payload = dict(**params)
        if "otp" not in payload:
            raise InvalidParameters("Otp not provided")
        if len(payload) > 1:
            raise InvalidParameters("This endpoint only accepts one body parameters")
        return self.post(f"{self.control_path}/disable_otp_finalize", payload)

    def enable_otp(self):
        """
        In the event that a customer wants to stop being able to complete transfers programmatically, this endpoint
        helps turn OTP requirement back on. No arguments required.
        """
        return self.post(f"{self.control_path}/enable_otp", params=None)


from exceptions.paystack_error import paystackError

class ServerError(paystackError):
    """
    Raised when the gateway raises an error.  Please contant support at support@getpaystack.com.

    See https://developers.paystackpayments.com/reference/general/exceptions/python#server-error
    """
    pass



from paystack.gateways.gateway import BaseGateway


class Plans(BaseGateway):
    gateway_path = "/plan"
    """
    The Plans API allows you create and manage installment payment options on your integration
    """

    def create(self, params):
        """
        Create a plan on your integration
        :param params: name (required) - Name of plan description - Short description of plan amount (required) -
        Amount to be charged in kobo interval (required) - Interval in words. Valid intervals are hourly, daily,
        weekly, monthly, annually. send_invoices - Set to false if you don't want invoices to be sent to your
        customers currency - Currency in which amount is set invoice_limit - Number of invoices to raise during
        subscription to this plan. Can be overridden by specifying an invoice_limit while subscribing.
        :return:
        """
        return self.post(self.gateway_path, params)

    def list(self, query_params=None):
        """
        List plans available on your integration.
        :param query_params: query params for filtering of results
        :return:
        """
        return self.get(self.gateway_path, query_params)

    def retrieve(self, id_or_code):
        """
        Get details of a plan on your integration.
        :param id_or_code: Plan ID or Code
        :return:
        """
        return self.get(f"{self.gateway_path}/{id_or_code}")

    def update(self, id_or_code, params):
        """
        Update a plan details on your integration
        :param id_or_code: Plan ID or Code
        :param params: same as with Plans.create(**params) method - update fields based on your needs
        :return:
        """
        return self.put(f"{self.gateway_path}/{id_or_code}", params)



from paystack.gateways.gateway import BaseGateway
from paystack.exceptions.card_error import CardError
from paystack.exceptions.bank_error import BankError
from paystack.exceptions.invalid_parameter import InvalidParameters


class Charge(BaseGateway):
    gateway_path = "/charge"
    """
    The Charge API allows you to configure payment channel of your choice when initiating a payment.\n The charge 
    API allows you to pass details of any payment instrument directly to Paystack, along with the transaction details 
    (email, amount, etc). We cover all the different payment methods in detail below, but in this section, 
    we want to discuss how to handle responses from the charge API. Here is a sample payload to the Charge API 
    containing transaction details and an object for a payment instrument.\n When you call the Charge API, 
    the response contains a data.status which tells you what the next step in the process. Depending on the value in 
    the data.status, you may need to prompt the user for an input as indicated in the response message (like OTP or 
    pin or date of birth), or display an action that the user needs to complete on their device - like scanning a QR 
    code or dialling a USSD code or redirecting to a 3DSecure page. So you follow the prompt on the data.status until 
    there is no more user input required, then you can call the verify endpoint to confirm payment. For the steps 
    that prompt for user input, you will be required to display a form to the user to collect the requested input and 
    send it to the relevant endpoint as shown in the table below. For the steps that require the user to complete an 
    action on their device, we recommend that you display a button for the user to confirm the payment after they 
    have performed that action so that you can call the verify endpoint to confirm the payment. 
    """

    def submit(self, url, params, obj):
        payload = dict(**params)
        ref = payload.get("reference", None)
        _obj = payload.get(obj, None)
        if not ref:
            raise InvalidParameters("Transaction reference cannot be null")
        if not _obj:
            raise InvalidParameters(f"{obj} cannot be null")
        return self.post(url, payload)

    def charge(self, params):
        """
        Send card details or bank details or authorization code to start a charge. Simple guide to charging cards
        directly \n
        :param params: email (required) - Customer's email address amount (required) - Amount in kobo
        card (required) - Card number card.number (required) - Card to tokenize card.cvv (required) - Card security
        code card.expiry_month (required) - Expiry month of card card.expiry_year (required) - Expiry year of card
        bank - Bank account to charge (don't send if charging an authorization code or card) bank.code (required) - A
        code for the bank (check banks for the banks supported). Only the ones for which paywithbank is true will
        work. bank.account_number (required) - 10 digit nuban for the account to charge authorization_code - An
        authorization code to charge (don't send if charging a card or bank account) pin - 4-digit PIN (send with a
        non-reusable authorization code) metadata - A JSON object \n
        :return:
        """
        return self.post(self.gateway_path, params)

    def charge_card(self, params):
        """
        Directly charge a bank card.\n We strongly discourage passing card information directly to the API to avoid
        transmitting card data through systems that are not PCI compliant. If you are PCI-DSS certified and would
        like to be able to send cardholder information directly to our APIs from your servers, reach out to us!.\n
        :param params:
        :return:
        """
        payload = dict(**params)
        if len(payload) == 0:
            raise InvalidParameters("Parameters cannot be empty")
        card = payload["card"]
        if len(card) == 0:
            raise CardError("Bank card have not been provided")
        if card["cvv"] == "":
            raise CardError("Invalid cvv")
        if card["expiry_year"] == "":
            raise CardError("Invalid expiry")
        return self.charge(payload)

    def bank_charge(self, params):
        """
        The Pay with Bank feature allows customers pay through internet banking portal or by providing their bank
        account number and authenticating using an OTP sent to their phone or email. This is different from Bank
        Transfers where customers transfer money into a bank account.\n To collect bank details, you would need to
        prompt the user to select their bank and enter their account number. To fetch the list of supported banks,
        make a GET request to the List BanksAPI endpoint, with the additional filter pay_with_bank=true. The banks
        can be listed in a dropdown or any other format that allows the user to easily pick their bank of choice.\n
        :param params:
        :return:
        """
        payload = dict(**params)
        bank = payload["bank"]
        if len(bank) == 0:
            raise BankError("Bank details not provided")
        if bank["code"] and bank["account_number"] == "":
            raise BankError("Invalid bank details")
        if bank["code"] or bank["account_number"] == "":
            raise BankError("Invalid bank details")
        return self.charge(payload)

    def ussd_charge(self, params):
        """
        This Payment method is specifically for Nigerian customers. Nigerian Banks provide USSD services that
        customers use to perform transactions, and we've integrated with some of them to enable customers complete
        payments. The Pay via USSD channel allows your Nigerian customers to pay you by dialling a USSD code on their
        mobile device. This code is usually in the form of * followed by some code and ending with #. The user is
        prompted to authenticate the transaction with a PIN and then it is confirmed. All you need to initiate a USSD
        charge is the customer email and the amount to charge. When the user pays, a response will be sent to your
        webhook. Hence, for this to work properly as expected, webhooks must be set up on your Paystack Dashboard.
        :param params:
        :return:
        """
        payload = dict(**params)
        code = payload["code"]
        if len(code) == 0 or code["ussd"] == "":
            raise InvalidParameters("Invalid ussd parameter")
        return self.charge(payload)

    def qr_code(self, params):
        """
        The QR option generates a QR code which allows customers to use their bank's mobile app to complete payments.
        We currently have only Visa QR option available. We'll have more options later. When the customer scans the
        code, they authenticate on their bank app to complete the payment. When the user pays, a response will be
        sent to your webhook. This means that you need to have webhooks set up on your Paystack Dashboard.
        :param params:
        :return:
        """
        payload = dict(**params)
        return self.charge(payload)

    def submit_otp(self, params):
        """
        otp (required) - OTP submitted by user
        reference (required) - reference for ongoing transaction.\n
        :param params:
        :return:
        """
        return self.submit(self.gateway_path + "/submit_otp", params, "otp")

    def submit_pin(self, params):
        return self.submit(self.gateway_path + "/submit_pin", params, "pin")

    def submit_phone(self, params):
        return self.submit(self.gateway_path + "/submit_phone", params, "phone")

    def submit_dob(self, params):
        return self.submit(self.gateway_path + "/submit_birthday", params, "birthday")

    def submit_address(self, params):
        payload = dict(**params)
        return self.post(self.gateway_path + "/submit_address", payload)

    def check_charge_status(self, reference):
        return self.get(self.gateway_path+"/"+reference)



from paystack.exceptions.invalid_parameter import InvalidParameters


class BaseGateway:
    gateway_path = ""

    def __init__(self, gateway):
        self.gateway = gateway
        self.config = gateway.config

    def post(self, url, params=None):
        if params is None:
            params = {}
        res = self.config.http().post(url, params)
        print(res)
        return res

    def get(self, url, query_params=None):
        if query_params is None:
            query_params = {}

        res = self.config.http().get(url, params=query_params)
        print(res)
        return res

    def put(self, url, params=None):
        if params is None:
            params = {}
        res = self.config.http().put(url, params)
        print(res)
        return res

    def delete(self, url):
        res = self.config.http().delete(url)
        print(res)
        return res

    def validate(self, fields, required=None):
        fields = dict(**fields)
        if required is None:
            required = []
        if len(fields) == 0:
            raise InvalidParameters("empty body cant be sent")
        for field in required:
            if field == "source":
                if "source" not in fields:
                    fields["source"] = "balance"
            if field not in fields:
                raise InvalidParameters(f"{field} is required")
            if fields[field] == "":
                raise InvalidParameters(f"{field} is required")
        return fields



from exceptions.paystack_error import paystackError


class AuthorizationError(paystackError):
    """
    Raised when the user does not have permission to complete the requested operation.
    """
    pass



from exceptions.unexpected_error import UnexpectedError


class TimeoutError(UnexpectedError):
    pass


class ConnectTimeoutError(TimeoutError):
    pass


class ReadTimeoutError(TimeoutError):
    pass



from paystack.gateways.gateway import BaseGateway


class TransactionSplit(BaseGateway):
    gateway_path = "/split"
    """

    The Transaction Splits API enables merchants split the settlement for a transaction across their payout account,
    and one or more Subaccounts.

    """

    def create(self, params):
        """
        Create a split payment on your integration
        :param params: 'name:"Name of the transaction split"' 'type: "The
        type of transaction split you want to create. You can use one of the following: percentage | flat"'
        'currency: "NGN, GHS, or USD"' 'subaccounts:"A list of object containing subaccount code and number of
        shares: [{subaccount_code: ‘ACT_xxxxxxxxxx’, share: xxx},{...}]"' 'bearer_type:"Any of subaccount | account |
        all"' 'bearer_subaccount:"Subaccount code"'
        """
        return self.post(self.gateway_path, params)

    def query(self, query_params=None):
        """
        List/search for the transaction splits available on your integration.
        :param query_params: name, active, sort_by, per_page, page, from, to
        """
        return self.get(self.gateway_path, query_params)

    def fetch(self, split_id):
        """
        Get details of a split on your integration.
        :param split_id: split id
        """
        return self.get(f"{self.gateway_path}/{split_id}")

    def update(self, split_id, params):
        """
        Update a transaction split details on your integration
        :param split_id: Split ID
        :param params: name, active -- check the documentation for optional parameters
        """
        return self.put(f"{self.gateway_path}/{split_id}", params)



import requests
import json

from exceptions import AuthenticationError, AuthorizationError, NotFoundError, RequestTimeoutError, \
    UpgradeRequiredError, TooManyRequestsError, ServerError, ServiceUnavailableError, GatewayTimeoutError, \
    UnexpectedError
from exceptions.http import InvalidResponseError
from exceptions.http.timeout_error import ConnectTimeoutError, ReadTimeoutError


class Http:

    def __init__(self, secret_key, timeout):
        self.secret_key = secret_key
        self.timeout = timeout
        self.base_url = "https://api.paystack.co"

    @staticmethod
    def is_error_status(status):
        return status not in [200, 201]

    @staticmethod
    def raise_exception_from_status(status, message=None):
        if status == 401:
            raise AuthenticationError()
        elif status == 403:
            raise AuthorizationError(message)
        elif status == 404:
            raise NotFoundError()
        elif status == 408:
            raise RequestTimeoutError()
        elif status == 426:
            raise UpgradeRequiredError()
        elif status == 429:
            raise TooManyRequestsError()
        elif status == 500:
            raise ServerError()
        elif status == 503:
            raise ServiceUnavailableError()
        elif status == 504:
            raise GatewayTimeoutError()
        else:
            raise UnexpectedError("Unexpected HTTP_RESPONSE " + str(status))

    def handle_exception(self, exception):
        if isinstance(exception, requests.exceptions.ReadTimeout):
            raise ReadTimeoutError(exception)
        elif isinstance(exception, requests.exceptions.ConnectTimeout):
            raise ConnectTimeoutError(exception)
        elif isinstance(exception, requests.exceptions.ConnectionError):
            raise ConnectionError(exception)
        elif isinstance(exception, requests.exceptions.HTTPError):
            raise InvalidResponseError(exception)
        elif isinstance(exception, requests.exceptions.Timeout):
            raise TimeoutError(exception)
        else:
            raise UnexpectedError(exception)

    class ContentType(object):
        json = "application/json"
        multipart = "multipart/form-data"

    def _headers(self, content_type):
        headers = {
            "Authorization": f"Bearer {self.secret_key}"
        }
        if content_type == Http.ContentType.json:
            headers["Content-Type"] = Http.ContentType.json
        return headers

    def _request_body(self, params, files):
        if files is None:
            return params
        return params, files

    def _request_function(self, method):
        if method == "GET":
            return requests.get
        elif method == "POST":
            return requests.post
        elif method == "PUT":
            return requests.put
        elif method == "DELETE":
            return requests.delete

    def _call_api(self, http_verb, path, headers, request_body, query_params=None):
        data = request_body
        files = None
        if query_params is None:
            query_params = {}
        if type(request_body) is tuple:
            data = request_body[0]
            files = request_body[1]

        response = self._request_function(http_verb)(
            self.base_url + path,
            headers=headers,
            json=data,
            files=files,
            params=query_params,
            timeout=self.timeout
        )

        return [response.status_code, response.json()]

    def _make_request(self, http_verb, path, content_type, params=None, files=None, query_params=None):
        response_body = None
        status = None
        headers = self._headers(content_type)
        request_body = self._request_body(params, files)
        full_path = path
        try:
            status, response_body = self._call_api(http_verb, full_path, headers, request_body, query_params)
        except Exception as e:
            self.handle_exception(e)
        print(response_body)
        if Http.is_error_status(status):
            Http.raise_exception_from_status(status)
        else:
            if len(response_body) == 0:
                return {}
            else:
                if content_type == Http.ContentType.json:
                    return request_body

    def post(self, path, params=None):
        return self._make_request("POST", path, Http.ContentType.json, params)

    def delete(self, path):
        return self._make_request("DELETE", path, Http.ContentType.json)

    def get(self, path, params=None):
        return self._make_request("GET", path, Http.ContentType.json, query_params=params)

    def put(self, path, params=None):
        return self._make_request("PUT", path, Http.ContentType.json, params)

    def post_multipart(self, path, files, params=None):
        return self._make_request("POST", path, Http.ContentType.multipart, params, files)



from paystack.gateways.gateway import BaseGateway
from paystack.exceptions.invalid_parameter import InvalidParameters


class Transfers(BaseGateway):
    gateway_path = "/transfer"
    recipient_path = "/transferrecipient"

    def add_beneficiary(self, params):
        """
        Creates a new recipient. A duplicate account number will lead to the retrieval of the existing record.\n
        :param params:  type (required) - Recipient Type (Only nuban at this time)
        name (required) - A name for the recipient
        metadata - Store additional information about your recipient in a structured format. JSON
        bank_code (required) - Required if type is nuban. You can find a list of bank codes at api.paystack.co/bank
        account_number (required) - Required if type is nuban
        currency - Currency for the account receiving the transfer.
        description
        """
        payload = dict(**params)
        if "type" not in payload:
            payload["type"] = "nuban"
        if "name" not in payload:
            raise InvalidParameters("Please specify the account name")
        if "account_number" not in payload:
            raise InvalidParameters("Please specify the account number")
        if "bank_code" not in payload:
            raise InvalidParameters("Bank code is required")
        return self.post(self.recipient_path, payload)

    def list_beneficiaries(self, query_params=None):
        return self.get(self.recipient_path, query_params)

    def update_beneficiary(self, code_or_id, params):
        payload = dict(**params)
        return self.put(f"{self.recipient_path}/{code_or_id}", payload)

    def remove_beneficiary(self, code_or_id):
        return self.delete(f"{self.recipient_path}/{code_or_id}")

    def initiate(self, params):
        """
        Status of transfer object returned will be ‘pending’ if OTP is disabled. In the event that an OTP is
        required, status will read ‘otp’.\n
        :param params: source (required) - Where should we transfer from? Only
        balance for now amount - Amount to transfer in kobo currency - NGN reason recipient (required) - Code for
        transfer recipient reference - If specified, the field should be a unique identifier (in lowercase) for the
        object. Only - , _ and alphanumeric characters allowed.
        """
        payload = dict(**params)
        if "source" not in payload:
            payload["source"] = "balance"
        if "amount" not in payload:
            return InvalidParameters("Please specify the amount you would like to transfer")
        if "recipient" not in payload:
            return InvalidParameters("Please include the recipient of this transfer")
        return self.post(self.gateway_path, payload)

    def list(self, query_params=None):
        return self.get(self.gateway_path, query_params)

    def verify(self, reference):
        return self.get(f"{self.gateway_path}/verify/{reference}")

    def fetch(self, id_or_code):
        return self.get(f"{self.gateway_path}/{id_or_code}")

    def finalize(self, params):
        """
        Finalizes a transfer
        :param params: transfer_code (required) - Transfer code
        otp (required) - OTP sent to business phone to verify transfer.
        """
        payload = dict(**params)
        if "transfer_code" not in payload:
            raise InvalidParameters("Transfer code is required")
        if "otp" not in payload:
            raise InvalidParameters("Otp is required")
        return self.post(f"{self.gateway_path}/finalize_transfer", payload)

    def initiate_bulk(self, params):
        """
        You need to disable the Transfers OTP requirement to use this endpoint.
        :param params: source - Where should we transfer from? Only balance for now
        transfers - Each object should contain amount, recipient, and reference
        """
        payload = dict(**params)
        self.validate(payload)
        if "source" not in payload:
            payload["source"] = "balance"
        if "transfers" not in payload:
            raise InvalidParameters("Transfers array must be provided")
        elif "transfers" in payload and not isinstance(payload["transfers"], list):
            raise InvalidParameters("Transfers must be an array of objects")
        return self.post(f"{self.gateway_path}/bulk", payload)


from exceptions.invalid_parameter import InvalidParameters
from paystack.gateways.gateway import BaseGateway


class Subscriptions(BaseGateway):
    gateway_path = "/subscription"

    """
    The Subscriptions API allows you create and manage recurring payment on your integration
    """

    def create(self, params):
        """
        Create a subscription on your integration \n
        :param params: customer (required) - Customer's email address or customer code
        plan (required) - Plan code
        authorization - If customer has multiple authorizations, you can set the desired authorization you wish to use for this subscription here. If this is not supplied, the customer's most recent authorization would be used
        start_date - Set the date for the first debit. (ISO 8601 format) \n
        Note the email_token attribute for the subscription object. We create one on each subscription so customers can
        cancel their subscriptions from within the invoices sent to their mailboxes. Since they are not authorized,
        the email tokens are what we use to authenticate the requests over the API.
        :return:
        """
        payload = dict(**params)
        if "customer" not in payload:
            raise InvalidParameters("Customer required")
        if "plan" not in payload:
            raise InvalidParameters("Plan required")
        return self.post(self.gateway_path, payload)

    def activate(self, params):
        """
        Enable a subscription on your integration \n
        :param params: code (required) - Subscription code
        token (required) - Email token

        :return:
        """
        payload = dict(**params)
        if "code" not in payload:
            raise InvalidParameters("Subscription code required")
        if "token" not in payload:
            raise InvalidParameters("Token required")
        return self.post(self.gateway_path + "/enable", payload)

    def deactivate(self, params):
        """
        Enable a subscription on your integration \n
        :param params: code (required) - Subscription code
        token (required) - Email token
        :return:
        """
        if "code" not in params:
            raise InvalidParameters("Subscription code required")
        if "token" not in params:
            raise InvalidParameters("Email token required")
        return self.post(self.gateway_path + "/disable", params)

    def list(self, query_params=None):
        """
        Disable a subscription on your integration \n
        :param query_params: perPage - Specify how many records you want to retrieve per page
        page - Specify exactly what page you want to retrieve
        customer - Filter by Customer ID
        plan - Filter by Plan ID
        :return:
        """
        return self.get(self.gateway_path, query_params)

    def retrieve(self, id_or_code):
        """
        Get details of a subscription on your integration. \n
        :param id_or_code: The subscription ID or code you want to fetch
        :return:
        """
        return self.get(f"{self.gateway_path}/{id_or_code}")



from paystack.exceptions.invalid_parameter import InvalidParameters
from paystack.gateways.gateway import BaseGateway


class Customer(BaseGateway):
    """
    The Customers API allows you create and manage customers on your integration.
    """

    def create(self, params):
        """
        Creates a customer
        :param params: name, first_name, last_name, phone, meta_data
        :return: response
        """
        return self.post("/customer", params)

    def list(self, query_params=None):
        """
        Returns the list of customers on your account
        :param query_params:
        :return:
        """
        return self.get("/customer", query_params)

    def fetch(self, email_or_code):
        """
        Get details of a customer on your integration.
        :param email_or_code: An email or customer code for the customer you want to fetch
        :return:
        """
        return self.get(f"/customer/{email_or_code}")

    def update(self, code, params):
        """
        Update a customer's details on your integration
        :param code: Customer's code
        :param params: first_name, last_name, phone, metadata
        :return:
        """
        return self.put(f"/customer/{code}", params)

    def set_risk_action(self, params):
        """
        Whitelist or blacklist a customer on your integration
        :param params: customer, risk_action - One of the possible risk actions [ default, allow, deny ]. allow to
        whitelist. deny to blacklist. Customers start with a default risk action.
        :return:
        """
        return self.post("/customer/set_risk_action", params)

    def deactivate_card(self, params):
        """
        Deactivate an authorization when the card needs to be forgotten
        :params: authorization_code - customer authorization code
        :return:
        """
        required_fields = ["authorization_code"]
        cleaned_data = self.validate(params, required_fields)
        return self.post("/customer/deactivate_authorization", cleaned_data)



from exceptions.authentication_error import AuthenticationError
from exceptions.authorization_error import AuthorizationError
from exceptions.configuration_error import ConfigurationError
from exceptions.gateway_timeout_error import GatewayTimeoutError
from exceptions.invalid_challenge_error import InvalidChallengeError
from exceptions.invalid_signature_error import InvalidSignatureError
from exceptions.not_found_error import NotFoundError
from exceptions.request_timeout_error import RequestTimeoutError
from exceptions.server_error import ServerError
from exceptions.service_unavailable_error import ServiceUnavailableError
from exceptions.test_operation_performed_in_production_error import TestOperationPerformedInProductionError
from exceptions.too_many_requests_error import TooManyRequestsError
from exceptions.unexpected_error import UnexpectedError
from exceptions.upgrade_required_error import UpgradeRequiredError



from exceptions.paystack_error import paystackError

class NotFoundError(paystackError):
    """
    Raised when an object is not found in the gateway, such as a Transaction.find("bad_id").

    See https://developers.paystackpayments.com/reference/general/exceptions/python#not-found-error
    """
    pass



from paystack.gateways.gateway import BaseGateway


class Settlements(BaseGateway):
    gateway_path = "/settlement"

    def load(self, query_params=None):
        """
        :param query_params: from - Lower bound of date range. Leave undefined to export settlement from day one. to
        - Upper bound of date range. Leave undefined to export settlements till date. subaccount - Provide a
        subaccount code to export only settlements for that subaccount. Set to none to export only transactions for
        the account
        """
        return self.get(self.gateway_path, query_params)

    def fetch_for_transactions(self, sid, query_params=None):
        """
        :param sid: settlement ID.\n
       :param query_params: from - Lower bound of date range. Leave undefined to export settlement from day one. to
       - Upper bound of date range. Leave undefined to export settlements till date. subaccount - Provide a
       subaccount code to export only settlements for that subaccount. Set to none to export only transactions for
       the account
       """
        return self.get(f"{self.gateway_path}/{sid}/transactions", query_params)



import paystack.configuration
import paystack.exceptions
import paystack.exceptions.http
import paystack.api.http
from paystack.gateways.transactions import TransactionGateway
from paystack.gateways.payment_splits import TransactionSplit
from paystack.gateways.subaccounts import Subaccount
from paystack.gateways.customers import Customer
from paystack.gateways.payment_plans import Plans
from paystack.gateways.Subscriptions import Subscriptions
from paystack.gateways.charge import Charge
from paystack.gateways.refund import Refunds
from paystack.gateways.transfer import Transfers
from paystack.gateways.controls import TransferControl
from paystack.gateways.misc import Misc
from paystack.gateways.settlements import Settlements


class PaystackGateway:

    def __init__(self, secret_key, public_key):
        self.config = paystack.configuration.Configuration(
            secret_key=secret_key,
            public_key=public_key
        )
        self.Transaction = TransactionGateway(self)
        self.Split = TransactionSplit(self)
        self.Customer = Customer(self)
        self.Subaccount = Subaccount(self)
        self.Plans = Plans(self)
        self.Subscriptions = Subscriptions(self)
        self.Charge = Charge(self)
        self.Refunds = Refunds(self)
        self.Transfers = Transfers(self)
        self.Control = TransferControl(self)
        self.Misc = Misc(self)
        self.Settlements = Settlements(self)


import paystack
from paystack.exceptions import ConfigurationError
import paystack.api.http


class Configuration:

    def __init__(self, secret_key, public_key, **kwargs):
        if secret_key == "":
            raise ConfigurationError("Missing Secret key")
        if public_key == "":
            raise ConfigurationError("Missing Public key")
        self.secret_key = secret_key
        self.public_key = public_key
        self.timeout = kwargs.get("timeout", 60)

    @staticmethod
    def configure(secret_key, public_key, **kwargs):
        Configuration.secret_key = secret_key
        Configuration.public_key = public_key
        Configuration.timeout = kwargs.get("timeout", 60)

    @staticmethod
    def instantiate():
        return Configuration(
            secret_key=Configuration.secret_key,
            public_key=Configuration.public_key,
            timeout=Configuration.timeout
        )

    @staticmethod
    def gateway():
        return paystack.PaystackGateway(config=Configuration.instantiate())

    def http(self):
        return paystack.api.http.Http(secret_key=self.secret_key, timeout=self.timeout)



from paystack.exceptions.invalid_parameter import InvalidParameters
from paystack.gateways.gateway import BaseGateway


class Refunds(BaseGateway):
    gateway_path = "/refund"

    """A Refund is an object created on a disputed transaction requesting for a refund. Credit/Debit card originally 
    charged will be refunded """

    def refund(self, params):
        """
        This creates a refund which is then processed by the Paystack team.\n
        :param params: transaction (required): Identifier for transaction to be refunded amount (optional): How much
        in kobo to be refunded to the customer. Amount is optional(defaults to original transaction amount) and
        cannot be more than the original transaction amount. currency: Three-letter ISO currency customer_note (
        optional): customer reason merchant_note (optional): merchant reason.\n
        """
        payload = dict(**params)
        if "transaction" not in payload:
            raise InvalidParameters("Transaction ID must be included")
        return self.post(self.gateway_path, payload)

    def list(self, query_params=None):
        """
        :param query_params: transaction, currency
        :return:
        """
        return self.get(self.gateway_path, query_params)

    def fetch(self, refund_id):
        """
        :param refund_id: Identifier of the refund object
        :return:
        """
        return self.get(self.gateway_path+"/"+refund_id)



from paystack.gateways.gateway import BaseGateway


class Misc(BaseGateway):
    gateway_path = "/bank"

    def list_banks(self, query_params=None):
        return self.get(self.gateway_path, query_params)

    def providers(self):
        query = {"pay_with_bank_transfer": True}
        return self.get(self.gateway_path, query_params=query)


from paystack.exceptions.invalid_parameter import InvalidParameters
from paystack.gateways.gateway import BaseGateway


class Subaccount(BaseGateway):
    """
    The Subaccounts API allows you create and manage subaccounts on your integration. Subaccounts can be used to
    split payment between two accounts (your main account and a sub account)
    """

    gateway_path = "/subaccount"

    def create(self, params):
        """
        Create a subacount on your integration
        :param params: business_name (required) - Name of business for subaccount settlement_bank (required) - Name
        of Bank (see list of accepted names by calling List Banks account_number (required) - NUBAN Bank Account
        Number percentage_charge (required) - What is the default percentage charged when receiving on behalf of this
        subaccount? primary_contact_email - A contact email for the subaccount primary_contact_name - A name for the
        contact person for this subaccount primary_contact_phone - A phone number to call for this subaccount
        metadata - Stringified JSON object settlement_schedule - Any of auto, weekly, monthly, manual. Auto means
        payout is T+1 and manual means payout to the subaccount should only be made when requested.
        :return:
        """
        payload = dict(**params)
        if "business_name" not in payload:
            raise InvalidParameters("Business name required")
        if "settlement_bank" not in payload:
            raise InvalidParameters("Settlement bank needs to be provided")
        if "account_number" not in payload:
            raise InvalidParameters("Account number required")
        if "percentage_charge" not in payload:
            raise InvalidParameters("Percentage charge must be fixed")
        return self.post(self.gateway_path, payload)

    def list(self, query_params=None):
        """
        List subaccounts available on your integration.
        :param query_params: perPage - Specify how many records you want to retrieve per page page - Specify exactly
        what page you want to retrieve - from A timestamp from which to start listing subaccounts e.g.
        2016-09-24T00:00:05.000Z, 2016-09-21 - to A timestamp at which to stop listing subaccounts e.g.
        2016-09-24T00:00:05.000Z, 2016-09-21
        :return:
        """
        return self.get(self.gateway_path, query_params)

    def retrieve(self, id_or_code):
        """
        Get details of a subaccount on your integration.
        :param id_or_code: Subaccount id or code
        :return:
        """
        return self.get(f"{self.gateway_path}/{id_or_code}")

    def update(self, id_or_code, params):
        """
        Update a subaccount details on your integration
        :param id_or_code: Subaccount id or code
        :param params: params for create method (update fields based on your needs)
        :return:
        """
        return self.put(f"{self.gateway_path}/{id_or_code}", params)



from exceptions.paystack_error import paystackError

class TestOperationPerformedInProductionError(paystackError):
    """
    Raised when an operation that should be used for testing is used in a production environment
    """
    pass



from exceptions.paystack_error import paystackError


class AuthenticationError(paystackError):
    """
    Raised when the client library cannot authenticate with the gateway.  This generally means the public_key/private key are incorrect, or the user is not active.
    """
    pass
